#! /usr/bin/env python

#Class to handle split alignments from a single contig
class AnomalousContigCls:
  #alignment1 in pair
  align1 = None
  #alignment1 in pair
  align2 = None
  #contig number
  contig = None
  #bp overlap in contig alignments
  overlap_query = None
  #fraction of alignments that overlap in contig coordinates
  overlap_query_fraction = None
  #bp's covered by the two alignments
  contig_cover = None
  #fraction of total contig covered by two alingments
  contig_cover_fraction = None
  # type of event: chimeric, exon duplication, local inversion
  type = None

  def __init__(self, a1, a2, type):
    #Read in the alignment pairs
    self.align1 = fix_align(a1)
    self.align2 = fix_align(a2)
    #attributes of the contig
    self.contig = a1.query
    self.contig_size = a1.query_len
    # set the type
    self.type = type
  # end def

  def target_distance(self):
    if (self.align1.target != self.align2.target):
      return "NA"
    # end if
    # sort target co-ordinates of the first block
    if (self.align1.tstart < self.align1.tend):
      start1 = self.align1.tstart
      end1 = self.align1.tend
    else:
      start1 = self.align1.tend
      end1 = self.align1.tstart
    # end if
    # sort target co-ordinates of the second block
    if (self.align2.tstart < self.align2.tend):
      start2 = self.align2.tstart
      end2 = self.align2.tend
    else:
      start2 = self.align2.tend
      end2 = self.align2.tstart
    # end if
    # sort the blocks by start co-ordinate
    if (start1 < start2):
      start1_s = start1
      end1_s = end1
      start2_s = start2
      end2_s = end2
    else:
      start1_s = start2
      end1_s = end2
      start2_s = start1
      end2_s = end1
    # end if
    td = int(start2_s) - int(end1_s)
    return td
  # end def

  def junction(self):
    # Order the alignments blocks in the split alignment by query start
    if (self.align1.qstart < self.align2.qstart):
      block1 = self.align1
      block2 = self.align2
    else:
      block1 = self.align2
      block2 = self.align1
    # end if

    # Determine the "junction_from" target co-ordinate
    junction_from = block1.tend
    # determine which side of the junction point block one represents
    if (block1.tstart < block1.tend):
      dir_from = "up"
    else:
      dir_from = "down"
    # end if
    #   BLAT: always orders the target co-ordinates as if the alignment
    #         was to the + strand
    #   Exonerate: orders the target co-ordinates in the actual direction
    #         of the alignment
    # If the alignment method was blat
    #if ("blat" == block1.method):
      # If the query strand was positive
      #if ("+" == block1.query_strand):
        # Use the target end coordinate from the first block
        #junction_from = block1.tend
        # block one represents the region upstream from the junction point
        #dir_from = "up"
      # If the query strand was negative
      #elif ("-" == block1.query_strand):
        # Use the target start coordinate from the first block
        #junction_from = block1.tstart
        # block one represents the region downstream from the junction point
        #dir_from = "down"
      #else:
        #msg = "Unrecognized query strand in blat alignment: %s" % \
          #block1.query_strand
        #raise SplitAlignmentError(msg)
      # end if
    # If the alignment method was exonerate
    #elif ("exonerate" == block1.method):
      # the target co-ordinates are ordered properly, so just
      # use the target end coordinate without checking strand
      #junction_from = block1.tend
      # determine which side of the junction point block one represents
      #if (block1.tstart < block1.tend):
        #dir_from = "up"
      #else:
        #dir_from = "down"
      # end if
    #else:
      #msg = "Unrecognized alignment method: %s" % \
        #block1.method
      #raise SplitAlignmentError(msg)
    # end if

    # Determine the "junction_to" target co-ordinate
    junction_to = block2.tstart
    # determine which side of the junction point block one represents
    if (block2.tstart < block2.tend):
      dir_to = "down"
    else:
      dir_to = "up"
    # end if
    # If the alignment method was blat
    #if ("blat" == block2.method):
      # If the query strand was positive
      #if ("+" == block2.query_strand):
        # Use the target start coordinate from the second block
        #junction_to = block2.tstart
        # block two represents the region downstream from the junction point
        #dir_to = "down"
      # If the query strand was negative
      #elif ("-" == block2.query_strand):
        # Use the target end coordinate from the second block
        #junction_to = block2.tend
        # block two represents the region upstream from the junction point
        #dir_to = "up"
      #else:
        #msg = "Unrecognized query strand in blat alignment: %s" % \
          #block2.query_strand
        #raise SplitAlignmentError(msg)
      # end if
    # If the alignment method was exonerate
    #elif ("exonerate" == block2.method):
      # the target co-ordinates are ordered properly, so just
      # use the target start coordinate without checking strand
      #junction_to = block2.tstart
      # determine which side of the junction point block one represents
      #if (block2.tstart < block2.tend):
        #dir_to = "down"
      #else:
        #dir_to = "up"
      # end if
    #else:
      #msg = "Unrecognized alignment method: %s" % \
        #block2.method
      #raise SplitAlignmentError(msg)
    # end if

    return "%s:%d(%s)-%s:%d(%s)" % \
      (block1.target, int(junction_from), dir_from,
       block2.target, int(junction_to),   dir_to)
  # end def

  def details(self):
    #function formats the output lines from the alignment pair
    ctg_fields = self.contig.split(" ")
    ctg = "%s(%sbp)" % (ctg_fields[0], self.contig_size)
    target = "%s:%s-%s,%s:%s-%s" % \
      (self.align1.target, self.align1.tstart, self.align1.tend,
       self.align2.target, self.align2.tstart, self.align2.tend)
    query = "%s-%s,%s-%s" % \
      (self.align1.qstart, self.align1.qend,
       self.align2.qstart, self.align2.qend)
    align_len1 = float(self.align1.qend) - float(self.align1.qstart) + 1
    align_len2 = float(self.align2.qend) - float(self.align2.qstart) + 1
    align_frac1 = align_len1/float(self.align1.query_len)
    align_frac2 = align_len2/float(self.align2.query_len)
    params = \
      "CO:%.3f,CC:%.2f,TD:%s,I1:%.1f,I2:%.1f,AF1:%.2f,AF2:%.2f" % \
      (self.overlap_query_fraction, self.contig_cover_fraction,
       self.target_distance(),
       float(self.align1.identity), float(self.align2.identity),
       align_frac1, align_frac2)
    detail_string = \
      "CTG:%s TYPE:%s TARGET:%s CONTIG:%s JUNCTION:%s %s" % \
      (ctg, self.type, target, query, self.junction(), params)
    return detail_string
  # end def
# end class

def fix_align(align):
  if ("chr" != align.target[0:3]):
    align.target = "chr%s" % align.target
  # end if
  if ("chrMT" == align.target):
    align.target = "chrM"
  # end if
  if ("blat" == align.method):
    if ("-" == align.query_strand):
      temp_tstart = align.tstart
      align.tstart = align.tend
      align.tend = temp_tstart
    # end if
    if (None != align.psl_str):
      PSL_TNAME_COL = 13
      psl_info = align.psl_str.split("\t")
      target_name = psl_info[PSL_TNAME_COL]
      if ("chr" != target_name[0:3]):
        psl_info[PSL_TNAME_COL] = "chr%s" % target_name
        align.psl_str = "\t".join(psl_info)
      # end if
    # end if
  elif ("exonerate" == align.method):
    if (align.target_strand):
      align.query_strand = "+"
    else:
      align.query_strand = "-"
    # end if
  # end if
  return align
# end def

#### EXCEPTION CLASSES ####
class SplitAlignmentError(Exception):
  def __init__(self, msg):
    self.msg = msg
  def __str__(self):
    return repr(self.msg)
# end class
