#include<iostream>
#include<fstream>
#include<sstream>
#include<string>
#include<cmath>
#include <iomanip>
using namespace std;

//GapBlatter
//Checks for different chimeric events by re-aligning gaps

// Duplication/Inv_Duplication -> Align the gap within the remaining contig
// Intronic_inversion -> align the gap to the the intron between adjacent blocks (reverse)
// Genic_inversion -> align the gap to the genic region between first and last blocks (reverse)
// Genic_rearrangement -> align the gap to the genic region between first and last blocks (same direction)

const string tempGapSeqFile = "__temp_Gap_Seq.fa";
const string tempQueryRefFile = "__temp_QRef.fa";
const string tempPSLOutputFile = "__temp_BLAT_Output.psl";
const string tempInvPSLOutputFile = "__temp_Inv_BLAT_Output.psl";
const string tempGenInvPSLOutputFile = "__temp_Genic_Inv_BLAT_Output.psl";
const string junkBLAT_CMDout = "__temp_BLAT_cmd_out.txt";
const string BlatConfigFile = "gapblatterWI.config"; //blat path "path=", blat options "options=", gap removal "gapRemove=", genome file 2bitGenPath=

//Command line parameters
int gapThreshold = 20; //any gap >=threshold 
double queryGapSimThreshold = 0.95;

//Config file parameters
string defaultBlatPath = "/home/pubseq/BioSw/blat/blat34/blat";
string defaultBlatOptions = "-noHead -stepSize=5 -repMatch=2253";
string default2bitGenPath = "/projects/validations/mm9/2bit/mm9.2bit";
int gapRemoveFLAG = 0; //By default it doesn't remove any additional gaps, if this is 1, all gaps are filled as N in queries.
string allGapRemovedQuery;
int printOoBFLAG = 0;

int externalSearchFLAG = 0;
int externalSearchLimit = 0;

int discardBLATmissFLAG = 0;
int separateMissLogFLAG = 0;
string missBLATLogFile = "extraBlatMisses.log";


//global data used by GapSearch and sub-procedures
int matches, mismatches, repmatches, junk, querySize, genStart, genEnd, genSize, blockCount, contigStart, contigEnd, qNumInsert; 
string junks,queryName, chrID, contigDirection;
int *blockSizes, *qIndices, *tIndices;

int ConvertQueryPos2GenPos(int queryPos, int& oobFlag)
{
	if(contigDirection == "-")
		queryPos = querySize - queryPos - 1;

	//Return the corresponding genome position of a query position
	for(int i=0; i<blockCount; i++)
	{
		if(queryPos < qIndices[i] + blockSizes[i])
		{
			if(queryPos < qIndices[i])
			{
				if(printOoBFLAG == 1)
					return -1;
			
				oobFlag = 1;

				// if printOoBFLAG is off then give the genome coordinates of the closest mapped base.
				if(i == 0)
				{
					return tIndices[0];
				}

				int difToBef = queryPos - (qIndices[i-1] + blockSizes[i-1]), difToAft = qIndices[i] - queryPos;
			
				if(difToBef < difToAft)
					return tIndices[i-1] + blockSizes[i-1] -1;
				else
					return tIndices[i];
			}		

			int offset = queryPos - qIndices[i];
			
			return tIndices[i] + offset;
		}
	}

	if(queryPos >= qIndices[blockCount-1] + blockSizes[blockCount-1] && queryPos < querySize)
	{
		if(printOoBFLAG == 1) // if printOoBFLAG is on simply return -1
			return -1;
		else 
		{
			oobFlag = 1;
			return tIndices[blockCount-1] + blockSizes[blockCount-1] - 1;
		}
	}

	// if queryPos >= querySize
	cout << "ERROR Q: query position is larger than query size" << endl;
	exit(251);
}

double calcPID(int qStart, int qEnd, int tStart, int tEnd, int qNumInsert, int qMatch, int qMisMatch, int qRepMatch)
{
	int milliBad = 0;

	int sizeMul = 1;
	int qAliSize, tAliSize, aliSize;
	int sizeDif;
	int insertFactor;
	int total;

	qAliSize = sizeMul * (qEnd - qStart);
	tAliSize = tEnd - tStart;
	aliSize = min(qAliSize, tAliSize);
	if (aliSize <= 0)
	    return 0;
	sizeDif = qAliSize - tAliSize;
	if (sizeDif < 0)
	    sizeDif = 0;

	insertFactor = qNumInsert;
	total = (sizeMul * (qMatch + qRepMatch + qMisMatch));
	if (total != 0)
	{
	    	int roundedValue = (int) round(3*log(1+sizeDif));
		milliBad = (1000 * (qMisMatch*sizeMul + insertFactor + roundedValue)) / total;
	}
	return 100.0 - (double)milliBad * 0.1 ;
}

double calcAF(int match, int mismatch, int repmatch, int size)
{
	return double(match + mismatch + repmatch) / double(size);
}

void CreateAllGapRemovedQuery(const string& query, int bCount, int* bSizes, int* bStarts, int qSize, string contigDirection)
{
	allGapRemovedQuery = query;

	if(contigDirection == "+")
	{
		for(int k=0; k<bStarts[0]; k++)
			allGapRemovedQuery.at(k) = 'N';
	
		for(int i=0; i<blockCount-1; i++)
		{
			int posGapStart = bStarts[i] + bSizes[i];
			if(posGapStart < bStarts[i+1])
			{
				int posGapEnd = bStarts[i+1];
				for(int k=posGapStart; k<posGapEnd; k++)
				{
					allGapRemovedQuery.at(k) = 'N';
				}
			}
		}	
	
		for(int k=bStarts[blockCount-1] + bSizes[blockCount-1]; k<qSize; k++)
			allGapRemovedQuery.at(k) = 'N';	
	}
	else
	{
		for(int k=qSize - bStarts[0]; k<qSize; k++)	//forst first gap
			allGapRemovedQuery.at(k) = 'N';
	
		for(int i=0; i<blockCount-1; i++)
		{
			int posGapStart = bStarts[i] + bSizes[i];
			if(posGapStart < bStarts[i+1])
			{
				int posGapEnd = bStarts[i+1];
				int removeStart = qSize - posGapEnd, removeEnd = qSize - posGapStart;
				for(int k=removeStart; k<removeEnd; k++)
				{
					allGapRemovedQuery.at(k) = 'N';
				}
			}
		}	
	
		for(int k=qSize - (bStarts[blockCount-1] + bSizes[blockCount-1]) - 1; k>=0; k--)	//for last gap
			allGapRemovedQuery.at(k) = 'N';	
	}
}

inline string OobOrInt(int flag)
{
	if(flag == 0)
		return "INT";
	else
		return "OoB";
}

int GapSearch(stringstream& str, const string& fullQuery, const string& outputFN, const string& outputDir)
{
	int numFoundMatch = 0;
	
	str >> matches >> mismatches >> repmatches >> junk >> qNumInsert >> junk >> junk >> junk >> contigDirection >> queryName >> querySize >> contigStart >> contigEnd >> chrID >> genSize >> genStart >> genEnd >> blockCount;
		
	blockSizes = (int*) malloc (blockCount*sizeof(int));
	qIndices = (int*) malloc (blockCount*sizeof(int));
	tIndices = (int*) malloc (blockCount*sizeof(int));
	
	string blockSizesStr, qIndicesStr, tIndicesStr;
	str >> blockSizesStr;
	str >> qIndicesStr;
	str >> tIndicesStr;

	int afterCommaIndex = 0;
	for(int i=0; i<blockCount; i++)
	{
		int nextCommaPos = blockSizesStr.find(",", afterCommaIndex);
		string tempStr = blockSizesStr.substr(afterCommaIndex, nextCommaPos - afterCommaIndex);
		blockSizes[i] = atoi(tempStr.c_str());
		afterCommaIndex = nextCommaPos + 1;
	}

	afterCommaIndex = 0;
	for(int i=0; i<blockCount; i++)
	{
		int nextCommaPos = qIndicesStr.find(",", afterCommaIndex);
		string tempStr = qIndicesStr.substr(afterCommaIndex, nextCommaPos - afterCommaIndex);
		qIndices[i] = atoi(tempStr.c_str());
		afterCommaIndex = nextCommaPos + 1;
	}

	afterCommaIndex = 0;
	for(int i=0; i<blockCount; i++)
	{
		int nextCommaPos = tIndicesStr.find(",", afterCommaIndex);
		string tempStr = tIndicesStr.substr(afterCommaIndex, nextCommaPos - afterCommaIndex);
		tIndices[i] = atoi(tempStr.c_str());
		afterCommaIndex = nextCommaPos + 1;
	}

	if(gapRemoveFLAG == 1)
		CreateAllGapRemovedQuery(fullQuery,blockCount,blockSizes,qIndices,querySize,contigDirection);

	double queryPIDinGen = calcPID(contigStart,contigEnd,genStart,genEnd,qNumInsert,matches,mismatches,repmatches);
	double alignFracQuery2Gen = calcAF(matches,mismatches,repmatches,querySize);

	for(int i=1; i<blockCount; i++)
	{
		int gapSize = qIndices[i] - (qIndices[i-1] + blockSizes[i-1]);
		if(gapSize >= gapThreshold)
		{
			int foundDuplicationFLAG = 0;

			//found gap - BLAT the gap within query
			int gapStartInd = qIndices[i-1] + blockSizes[i-1]; //for contigDirection == +
			if(contigDirection == "-")			
				gapStartInd = fullQuery.length() - gapSize - gapStartInd;

			string gapSeq = fullQuery.substr(gapStartInd, gapSize);
			
			ofstream foutGap((outputDir + tempGapSeqFile).c_str());

			foutGap << ">GapSeq-" << queryName << ":[" << gapStartInd << "-" << gapStartInd + gapSize - 1 << "]" << endl;
			foutGap << gapSeq << endl;

			if(gapSize <= matches + mismatches) //Strict Filtering, exclusively for duplication events (Don't look for duplication if there aren't enough matches to align the gap)
			{	
				//Create a new string with Ns in place of the gap
				string newQuery;
				if(gapRemoveFLAG == 0)
				{
					string Ns(gapSeq.length(),'N');
					newQuery = fullQuery.substr(0,gapStartInd) + Ns + fullQuery.substr( gapStartInd+gapSize, fullQuery.length() - (gapStartInd + gapSize));
				}
				else
					newQuery = allGapRemovedQuery;
				
				ofstream foutRef((outputDir + tempQueryRefFile).c_str());
				foutRef << ">Query_ref-" << queryName << endl << newQuery << endl;
	
				string systemCallLine = defaultBlatPath + " " + defaultBlatOptions + " " + outputDir + tempQueryRefFile + " " + outputDir + tempGapSeqFile + " " + outputDir + tempPSLOutputFile + " >> " + outputDir + junkBLAT_CMDout;
				system(systemCallLine.c_str());
	
				ifstream finPSL((outputDir + tempPSLOutputFile).c_str());
				int gMatch;
				while(finPSL >> gMatch)
				{
	
					int gMismatch,gRepmatch,ijunk,gapNumInsert,gapWithinStart,gapWithinEnd; string ijunks;
					finPSL >> gMismatch >> gRepmatch >> ijunk >> gapNumInsert >> ijunk >> ijunk >> ijunk;
					string direction;
					finPSL >> direction >> ijunks >> ijunk >> gapWithinStart >> gapWithinEnd >> ijunks >> ijunk;
									
					int dupPosStart, dupPosEnd;
					finPSL >> dupPosStart >> dupPosEnd;
		
					int iBC; string iBLs, iQSs, iTSs; //num blocks, block lengths, query block start indices, target block start indices.
					finPSL >> iBC >> iBLs >> iQSs >> iTSs;
	
					if(gMatch >= queryGapSimThreshold * gapSize)
					{
						foundDuplicationFLAG = 1;
							
						ofstream foutOut;
						foutOut.open(outputFN.c_str(),ios_base::app);
	
						foutOut << "CTG:" << queryName << "(" << querySize << "bp) ";
	
						foutOut << "TYPE:";  					
						if(direction == "+")
							foutOut << "gap-duplication ";
						else if(direction == "-")
							foutOut << "gap-inverted_duplication ";
						else
						{
							cout << "ERROR: unknown direction character " << endl;
							exit(253);
						}
		
						// Within BLAT PSL output, first base starts from 0, and for every interval (Start pos is the position of the first base, but End pos is right after the last base)
						// Furthermore the output file should consider start base as 1, and start-end positions should show inclusive interval boundaries.
						
						int oobLeftFlag=0,oobRightFlag=0;
						int gap2genStart = ConvertQueryPos2GenPos(dupPosStart,oobLeftFlag), gap2genEnd = ConvertQueryPos2GenPos(dupPosEnd-1,oobRightFlag);	// gap2genEnd is also decremented by 1
						///Note that if contig Direction is '-', this conversion will be handled within the function by using corresponding dupPosStart & dupPosEnd coordinates	

						if(gap2genStart==-1)
						{
							if(gap2genEnd==-1)
								foutOut << "TARGET:" << chrID << ":" << genStart+1 << "-" << genEnd << "," << chrID << ":" << "OoB" << ":" << "OoB" << " ";
							else
								foutOut << "TARGET:" << chrID << ":" << genStart+1 << "-" << genEnd << "," << chrID << ":" << "OoB" << ":" << gap2genEnd+1 << " ";
						}
						else if(gap2genEnd==-1)
						{
							foutOut << "TARGET:" << chrID << ":" << genStart+1 << "-" << genEnd << "," << chrID << ":" << gap2genStart+1 << ":" << "OoB" << " ";
						}
						else
							foutOut << "TARGET:" << chrID << ":" << genStart+1 << "-" << genEnd << "," << chrID << ":" << gap2genStart+1 << "-" << gap2genEnd+1 << " ";
					
						foutOut << "CONTIG:" << gapStartInd+1 << "-" << gapStartInd + gapSize << "," << dupPosStart+1 << "-" << dupPosEnd << " ";
	
						foutOut << "JUNCTION:" << chrID << ":" << gap2genStart+1 << "(down)-" << chrID <<":" <<gap2genEnd+1 << "(up)" << " ";
	
						double gapPIDinQuery = calcPID(gapWithinStart,gapWithinEnd,dupPosStart,dupPosEnd,gapNumInsert,gMatch,gMismatch,gRepmatch);
						double alignFracGap2Query = calcAF(gMatch,gMismatch,gRepmatch,querySize);
	
						char sstr[100];
						sprintf(sstr,"I1:%.1f,I2:%.1f,AF1:%.2f,AF2:%.2f ", queryPIDinGen, gapPIDinQuery, alignFracQuery2Gen, alignFracGap2Query);
						foutOut << sstr;
	
						///Block info indices for duplication events show positions within the contig
						double simScore = (double) gMatch / (double) gapSize;

						foutOut << "BLOCK:" << direction << ":" << genStart << ":" << genEnd << ":" << iBC << ":" << iBLs << ":" << iQSs << ":" << iTSs << ":" << simScore << ":" << OobOrInt(oobLeftFlag) << ":" << OobOrInt(oobRightFlag) << endl;
	
						numFoundMatch++;
					}
				}
				
				string removeCall = "rm " + outputDir + tempQueryRefFile;
				system(removeCall.c_str());
				removeCall = "rm " + outputDir + tempPSLOutputFile;
				system(removeCall.c_str());
			}

			if(foundDuplicationFLAG == 0)
			{
				// Look for inversion Events. (all for internal gaps. External gaps can be found by fusion search)
				// Firstly, in the local intronic search (only internal gaps are searched - first and last gaps aren't searched since there is no intronic boundary)
				if(i>0 && i<blockCount-1)
				{
					int foundIntronicInversionFLAG = 0;

					int intrIndexStart = tIndices[i-1] + blockSizes[i-1];
					int intrIndexEnd = tIndices[i];	//end index should be +1 than actual last index than can be matched
					
					if(intrIndexEnd - intrIndexStart >= gapSize) //The intron to search should be larger
					{
						stringstream refWithIndices;
						refWithIndices << default2bitGenPath << ":" << chrID << ":" << intrIndexStart << "-" << intrIndexEnd;
						string invSystemCallLine = defaultBlatPath + " " + defaultBlatOptions + " " + refWithIndices.str() + " " + outputDir + tempGapSeqFile + " " + outputDir + tempInvPSLOutputFile + " >> " + outputDir + junkBLAT_CMDout;
						system(invSystemCallLine.c_str());
			
						ifstream finInvPSL((outputDir + tempInvPSLOutputFile).c_str());
						int invMatch;
						while(finInvPSL >> invMatch)
						{
							int invMismatch,invRepmatch,invJunk,invGapNumInsert,invGapWithinStart,invGapWithinEnd; string invJunks;
							finInvPSL >> invMismatch >> invRepmatch >> invJunk >> invGapNumInsert >> invJunk >> invJunk >> invJunk;
							string invDirection;
							finInvPSL >> invDirection >> invJunks >> invJunk >> invGapWithinStart >> invGapWithinEnd >> invJunks >> invJunk;
												
							int invPosStart, invPosEnd;
							finInvPSL >> invPosStart >> invPosEnd;
				
							int invBC; string invBLs, invQSs, invTSs; //num blocks, block lengths, query block start indices, target block start indices.
							finInvPSL >> invBC >> invBLs >> invQSs >> invTSs;
			
							if(invMatch >= queryGapSimThreshold * gapSize)
							{
								ofstream foutOut;
								//cout << "sepFLAG: " << separateMissLogFLAG << endl;
								if(invDirection == contigDirection && (discardBLATmissFLAG || separateMissLogFLAG))
								{
									if(discardBLATmissFLAG)
									{
										foundIntronicInversionFLAG = 1;
										continue;
									}
									else //seperateMissLogFLAG is ON
										foutOut.open(missBLATLogFile.c_str(),ios_base::app);
								}
								else //not blat miss or normal output for all
									foutOut.open(outputFN.c_str(),ios_base::app);
								
								foutOut << "CTG:" << queryName << "(" << querySize << "bp) ";
			
								foutOut << "TYPE:";  					
								
								if(invDirection == contigDirection)
									foutOut << "gap-intronic_BLAT_miss ";
								else
									foutOut << "gap-intronic_inversion ";
			
								// Within BLAT PSL output, first base starts from 0, and for every interval (Start pos is the position of the first base, but End pos is right after the last)
								// Furthermore the output file should consider start base as 1, and start-end positions should show inclusive interval boundaries.
			
								int gap2genStart = invPosStart + intrIndexStart, gap2genEnd = invPosEnd + intrIndexStart;
									
								foutOut << "TARGET:" << chrID << ":" << genStart+1 << "-" << genEnd << "," << chrID << ":" << gap2genStart+1 << "-" << gap2genEnd << " ";
								
								foutOut << "CONTIG:" << gapStartInd+1 << "-" << gapStartInd + gapSize << "," << invGapWithinStart + gapStartInd + 1 << "-" << invGapWithinEnd + gapStartInd << " ";
				
								foutOut << "JUNCTION:" << chrID << ":" << gap2genStart+1 << "(down)-" << chrID <<":" << gap2genEnd << "(up)" << " ";
			
								double invGapPIDinQuery = calcPID(invGapWithinStart,invGapWithinEnd,invPosStart,invPosEnd,invGapNumInsert,invMatch,invMismatch,invRepmatch);
								double invAlignFracGap2Query = calcAF(invMatch,invMismatch,invRepmatch,querySize);
			
								char sstr[100];
								sprintf(sstr,"I1:%.1f,I2:%.1f,AF1:%.2f,AF2:%.2f ", queryPIDinGen, invGapPIDinQuery, alignFracQuery2Gen, invAlignFracGap2Query);
								foutOut << sstr;
								
								///Block info for intron-wide inversion events show positions within cropped genome (intron within neighbour blocks)
								double invSimScore = (double) invMatch / (double) gapSize;
								foutOut << "BLOCK:" << invDirection << ":" << intrIndexStart << ":" << intrIndexEnd << ":" << invBC << ":" << invBLs << ":" << invQSs << ":" << invTSs << ":" << invSimScore << endl;
			
								numFoundMatch++;
								foundIntronicInversionFLAG = 1;
							}
						}
						string removeCall = "rm " + outputDir + tempInvPSLOutputFile;
						system(removeCall.c_str());
					}
	
					// Search within whole gene
					// If NO external search, look within whole genic interval (between first and last blocks)
					// If external search wanted, look within genic interval + external interval on both sides. (between firstBlockBegin-ext and lastBlockEnd+ext)
					if(foundIntronicInversionFLAG == 0)
					{
						int genicIntrIndexStart, genicIntrIndexEnd;
						
						if(externalSearchFLAG == 0)
						{
							genicIntrIndexStart = tIndices[0] + blockSizes[i-1];
							genicIntrIndexEnd = tIndices[blockCount-1];	//end index should be +1 than actual last index than can be matched
						}
						else
						{
							genicIntrIndexStart = tIndices[0] - externalSearchLimit;
							if(genicIntrIndexStart < 0) 
								genicIntrIndexStart = 0;
							
							genicIntrIndexEnd = tIndices[blockCount-1] + blockSizes[blockCount-1] + externalSearchLimit;
							if(genicIntrIndexEnd > genSize)
								genicIntrIndexEnd = genSize;
						}

						stringstream refWithIndices;
						refWithIndices << default2bitGenPath << ":" << chrID << ":" << genicIntrIndexStart << "-" << genicIntrIndexEnd;
						string invSystemCallLine = defaultBlatPath + " " + defaultBlatOptions + " " + refWithIndices.str() + " " + outputDir + tempGapSeqFile + " " + outputDir + tempGenInvPSLOutputFile + " >> " + outputDir + junkBLAT_CMDout;
						system(invSystemCallLine.c_str());
						
						ifstream finGenInvPSL((outputDir + tempGenInvPSLOutputFile).c_str());
						
						int genInvMatch;
						while(finGenInvPSL >> genInvMatch)
						{
							int genInvMismatch,genInvRepmatch,genInvJunk,genInvGapNumInsert,genInvGapWithinStart,genInvGapWithinEnd; string genInvJunks;
							finGenInvPSL >> genInvMismatch >> genInvRepmatch >> genInvJunk >> genInvGapNumInsert >> genInvJunk >> genInvJunk >> genInvJunk;
							string genInvDirection;
							finGenInvPSL >> genInvDirection >> genInvJunks >> genInvJunk >> genInvGapWithinStart >> genInvGapWithinEnd >> genInvJunks >> genInvJunk;
												
							int genInvPosStart, genInvPosEnd;
							finGenInvPSL >> genInvPosStart >> genInvPosEnd;
				
							int genInvBC; string genInvBLs, genInvQSs, genInvTSs; //num blocks, block lengths, query block start indices, target block start indices.
							finGenInvPSL >> genInvBC >> genInvBLs >> genInvQSs >> genInvTSs;
			
							if(genInvMatch >= queryGapSimThreshold * gapSize)
							{
								ofstream foutOut;
								foutOut.open(outputFN.c_str(),ios_base::app);
			
								foutOut << "CTG:" << queryName << "(" << querySize << "bp) ";
			
								foutOut << "TYPE:";  					
								
								if(externalSearchFLAG == 0)
								{
									if(genInvDirection == contigDirection)
										foutOut << "gap-genic_rearrangement ";
									else
										foutOut << "gap-genic_inversion ";
								}
								else
								{
									if(genInvDirection == contigDirection)
										foutOut << "gap-genic_rearrangement(ext=" << externalSearchLimit << ") ";
									else
										foutOut << "gap-genic_inversion_(ext=" << externalSearchLimit << ") ";
								}

								// Within BLAT PSL output, first base starts from 0, and for every interval (Start pos is the position of the first base, but End pos is right after the last)
								// Furthermore the output file should consider start base as 1, and start-end positions should show inclusive interval boundaries.
			
								int gap2genStart = genInvPosStart + genicIntrIndexStart, gap2genEnd = genInvPosEnd + genicIntrIndexStart;
								
								foutOut << "TARGET:" << chrID << ":" << genStart+1 << "-" << genEnd << "," << chrID << ":" << gap2genStart+1 << "-" << gap2genEnd << " ";
								foutOut << "CONTIG:" << gapStartInd+1 << "-" << gapStartInd + gapSize << "," << genInvGapWithinStart + gapStartInd + 1 << "-" << genInvGapWithinEnd + gapStartInd << " ";
								foutOut << "JUNCTION:" << chrID << ":" << gap2genStart+1 << "(down)-" << chrID <<":" << gap2genEnd << "(up)" << " ";
			
								double genInvGapPIDinQuery = calcPID(genInvGapWithinStart,genInvGapWithinEnd,genInvPosStart,genInvPosEnd,genInvGapNumInsert,genInvMatch,genInvMismatch,genInvRepmatch);
								double genInvAlignFracGap2Query = calcAF(genInvMatch,genInvMismatch,genInvRepmatch,querySize);
			
								char sstr[100];
								sprintf(sstr,"I1:%.1f,I2:%.1f,AF1:%.2f,AF2:%.2f ", queryPIDinGen, genInvGapPIDinQuery, alignFracQuery2Gen, genInvAlignFracGap2Query);
								foutOut << sstr;
								
								///Note that block info for gene wide inversion events show events within cropped genome (intron within first & last blocks)
								double genInvSimScore = (double) genInvMatch / (double) gapSize;
								foutOut << "BLOCK:" << genInvDirection << ":" << genicIntrIndexStart << ":" << genicIntrIndexEnd << ":" << genInvBC << ":" << genInvBLs << ":" << genInvQSs << ":" << genInvTSs << ":" << genInvSimScore << endl;
			
								numFoundMatch++;
							}
						}
						string removeCall = "rm " + outputDir + tempGenInvPSLOutputFile;
						system(removeCall.c_str());
					}
				}
			}
			string removeCall = "rm " + outputDir + tempGapSeqFile;
			system(removeCall.c_str());
		}
	}	
	return numFoundMatch;
}

void CheckAndUpdateConfig() //Reads a config file and replaces with the default path and options if they exist.
{
	ifstream fin(BlatConfigFile.c_str());
	if(fin.is_open())
	{
		string line;
		while(getline(fin,line))
		{		
			if(line.length()>5 && line.substr(0,5) == "path=")
				defaultBlatPath = line.substr(5,line.length()-5);
			if(line.length()>8 && line.substr(0,8) == "options=")
				defaultBlatOptions = line.substr(8,line.length()-8);
			if(line.length()>10 && line.substr(0,10) == "gapRemove=")
				gapRemoveFLAG = line.at(10) - '0';
			if(line.length()>9 && line.substr(0,9) == "printOoB=")
				printOoBFLAG = line.at(9) - '0';
			if(line.length()>12 && line.substr(0,12) == "2bitGenPath=")
				default2bitGenPath = line.substr(12,line.length()-12);
			if(line.length()>10 && line.substr(0,10) == "extSearch=")
				externalSearchFLAG = line.at(10) - '0';
			if(line.length()>15 && line.substr(0,15) == "extSearchLimit=")
			{
				stringstream ss(line.substr(15,line.length()-15));
				ss >> externalSearchLimit;
			}
			if(line.length()>12 && line.substr(0,12) == "discardMiss=")
				discardBLATmissFLAG = line.at(12) - '0';
			if(line.length()>11 && line.substr(0,11) == "sepMissLog=")
				separateMissLogFLAG = line.at(11) - '0';
			if(line.length()>12 && line.substr(0,12) == "missLogName=")
				missBLATLogFile = line.substr(12,line.length()-12);
		}
	}
}

int main(int argc, char *argv[])
{
        if (1 < argc && string("-v") == string(argv[1]))
        {
          cout << "GapBlatterWI Version: 1.0" << endl;
          exit(0);
        }

	if(argc < 4)
	{
		cout << "ERROR: There are 3 required arguments -> 1[PSL_LINE] 2[CONTIG_SEQ] 3[OUTPUT_FILENAME]" << endl;
		cout << "       Optional arguments: 4[int(gapThreshold)] 5[double(similarity_Score_threshold)]" << endl;
		exit(250);
	}

	stringstream str(argv[1]);
	string fullQuery(argv[2]);
	string outputFileName(argv[3]);
        size_t last_slash_pos = outputFileName.find_last_of("/");
        string outputDir(outputFileName.substr(0,last_slash_pos+1));
        //cout << "\"" << outputDir << "\"" << endl;

	if(argc > 4)
	{
		int inpGapThreshold = atoi(argv[4]);
		if(inpGapThreshold>0)
		{
			gapThreshold = inpGapThreshold;
		}
	}	
	if(argc > 5)
	{
		double inpQueryGapSimThreshold = atof(argv[5]);
		if(inpQueryGapSimThreshold>0)
			queryGapSimThreshold = inpQueryGapSimThreshold;
	}
	
	CheckAndUpdateConfig();

	int numFound = GapSearch(str, fullQuery, outputFileName, outputDir);

	if(numFound > 100)
		numFound = 100; //Don't return anything larger than 100, to leave the rest for error exits.

	exit(numFound);

	return 0;

}
