from utilities.intspan import subsume, overlap, subtract, cardinality
from utilities.tools import flatten, reverse_complement

class Event:

    event_type = None
    contig = None
    transcript = None
    model = None
    exons = []
    chrom = None
    exon_coords = []
    align_blocks = []
    align_coords = []
    txt_start = None
    txt_end = None
    novel = True
    num_blocks = None
    num_exons = None
    x3 = None
    artefact = False
    valid_orf = False
    match_result = False
    splice_motives = False
    
    def __init__(self, event_type):
        self.event_type = event_type
        pass

    def details(self):
        details = "%s %s %s " % (self.event_type, self.contig, self.transcript)
        details += ",".join([str(e) for e in self.exons])
        details += " " + ",".join([str(b) for b in self.align_blocks])
        details += " " + self.coordinate()

        if self.match_result and self.match_result.align and self.match_result.align.orient:
            details += " " + self.match_result.align.orient

        if self.event_type == "retained_intron":
            if self.x3:
                details += " 3x"

            details += " " + self.ri_coord()

        elif self.event_type == "AS3":
            #details += " " + self.splice_sites
            ss = self.get_splice_sites(self.align_blocks[0], right=True)
            if ss:
                details += " " + ss[0]

        elif self.event_type == "AS5":
            ss = self.get_splice_sites(self.align_blocks[0], left=True)
            if ss:
                details += " " + ss[0]
            
        elif self.event_type == "novel_exon" or self.event_type == "AS53":
            sizes = []
            for block in self.align_coords:
                sizes.append(str(cardinality(block)))

            details += " " + ",".join(sizes)

            splice_sites = []
            for block in self.align_blocks:
                ss = ",".join(self.get_splice_sites(block, left=True, right=True))
                if ss:
                    splice_sites.append(ss)
            if splice_sites:
                details += " " + ";".join(splice_sites)

        elif self.event_type == "novel_utr":
            if self.match_result and self.match_result.align:
                loc = "?"
                
                if self.match_result.align.orient:
                    if self.match_result.align.orient == "+":
                        if 1 in self.align_blocks:
                            loc = "5'"
                        else:
                            loc = "3'"
                    elif self.match_result.align.orient == "-":
                        if 1 in self.align_blocks:
                            loc = "3'"
                        else:
                            loc = "5'"

                details += " " + loc

        if self.valid_orf:
            details += " orf:" + self.valid_orf

        return details

    def ri_coord(self):
        if not self.match_result:
            print "no match result", self.contig, self.transcript
            return ""
        
        coords_dict = {}
        for i in range(len(self.exons)):
            coords_dict[self.exons[i]] = self.exon_coords[i]

        bounds = []
        for i in range(len(self.exons)-1):
            e = self.exons[i]
            coord = self.exon_coords[i]
            next_coord = self.exon_coords[i+1]

            #should have only one alignment block
            if i == 0 and int(self.align_coords[0][0]) < int(coord[0]) and e > 1:
                #print e, self.txt.exons, self.align_coords
                if int(self.align_coords[0][0]) > int(self.txt.exons[e-2][1]):
                    bound = ",".join((str(self.txt.exons[e-2][0]), str(self.txt.exons[e-2][1])))
                    bound += ",<" + str(self.align_coords[0][0]) + ","
                    bound += ",".join((str(coord[0]), str(coord[1])))
                    bounds.append(bound)

            bound = ",".join((str(coord[0]), str(coord[1])))
            bound += ",--,"
            bound += ",".join((str(next_coord[0]), str(next_coord[1])))
            bounds.append(bound)

        e = self.exons[-1]
        coord = self.exon_coords[-1]
        if int(self.align_coords[0][1]) > int(coord[1]) and e < len(self.txt.exons):
            if int(self.align_coords[0][1]) < int(self.txt.exons[e][0]):
                bound = ",".join((str(coord[0]), str(coord[1])))
                bound += ",>" + str(self.align_coords[0][1]) + ","
                bound += ",".join((str(self.txt.exons[e][0]), str(self.txt.exons[e][1])))
                bounds.append(bound)

        return ";".join(bounds)


    def get_splice_sites(self, block, left=False, right=False):
        ss = []
        
        align = self.match_result.align

        #print "test", block, left, right, len(align.blocks), align.splice_sites
        
        if align and int(block) >= 0 and align.splice_sites and (left or right):
            splice_sites = align.splice_sites

            if splice_sites[0] == '':
                splice_sites.pop(0)

            #print "ss", splice_sites

            if left and int(block) > 1:
                splice_site = splice_sites[block-2]

                if align.orient and align.orient == '-':
                    splice_site = reverse_complement(splice_site)

                motif = "?"
                if self.splice_motives.has_key(splice_site):
                    motif = self.splice_motives[splice_site]

                donor = splice_site[:2]
                acceptor = splice_site[-2:]

                if align.orient and align.orient == '-':
                    donor = donor.upper()
                    acceptor = acceptor.lower()
                else:
                    donor = donor.lower()
                    acceptor = acceptor.upper()

                if donor and acceptor:
                    ss.append("-".join([donor, acceptor]) + "(%s)" % (motif))

                #print "left:",block,donor,acceptor

            if right and int(block) < len(align.blocks):
                splice_site = splice_sites[block-1]
                #print "right", splice_site

                if align.orient and align.orient == '-':
                    splice_site = reverse_complement(splice_site)

                motif = "?"
                if self.splice_motives.has_key(splice_site):
                    motif = self.splice_motives[splice_site]

                donor = splice_site[:2]
                acceptor = splice_site[-2:]

                if align.orient and align.orient == '-':
                    donor = donor.lower()
                    acceptor = acceptor.upper()
                else:
                    donor = donor.upper()
                    acceptor = acceptor.lower()

                #print "right:",block,donor,acceptor

                if donor and acceptor:
                    ss.append("-".join([donor, acceptor]) + "(%s)" % (motif))

        return ss

                    
    def coordinate(self):
        coord = ""
        
        if self.event_type == "skipped_exon" or self.event_type == "novel_intron" or self.event_type == "retained_intron":
            coord = self.chrom + ":" + str(self.exon_coords[0][0]) + "-" + str(self.exon_coords[-1][1])
        elif self.event_type == "novel_exon" or self.event_type == "novel_utr" or self.event_type == "novel_transcript" or self.event_type == "AS53":
            coord = self.chrom + ":" + str(self.align_coords[0][0]) + "-" + str(self.align_coords[-1][1])
        elif self.event_type == "AS5" or self.event_type == "AS3":
            coord = self.chrom + ":" + str(self.align_coords[0]) + "-" + str(self.align_coords[-1])

        return coord

    def multiple_3(self, apart):
        if apart % 3 == 0:
            return True
        else:
            return False

    def screen_orfs(self, orfs):
        print orfs[0]
        
        if self.match_result and self.match_result.align:
            align = self.match_result.align

            #identify correct orfs by matching flanking exons
            if self.event_type == "novel_exon" or self.event_type == "AS53":
                flank_block = None
                if align.orient == "+":
                    flank_block = self.align_blocks[0] - 1
                elif align.orient == "-":
                    flank_block = self.align_blocks[-1] + 1

                screened_orfs = []
                if flank_block >= 1 and flank_block <= len(align.blocks):
                    flank_coords = align.query_blocks[flank_block - 1]
                    flank_coords.sort(lambda x,y: int(x)-int(y))
                    bases = align.contig.sequence[int(flank_coords[0]):int(flank_coords[1])+1]

                    print "flank coords", align.query, flank_coords, bases
                    flank_orfs = find_orfs2(bases, 1, align.orient)

                    print "flank orfs", flank_orfs

                    for orf in orfs:
                        for flank_orf in flank_orfs:
                            if flank_orf in orf:
                                print "correct orf", align.query, orf
                                screened_orfs.append(orf)
                    
                    
            

            #determine if orf pass thru event

    def test_orf(self, orf):
        #map coordinates of events to contig coorindates
        #print self.match_result.txt_id, self.match_result.align_id, self.match_result.align

        if self.match_result and self.match_result.align:
            align = self.match_result.align
            #print align.query, align.query, self.align_blocks
            if align.qstart and self.align_blocks:
                span_sum = 0
                qstart = int(self.match_result.align.qstart)
                for i in range(0, self.align_blocks[0]-1):
                    span = int(align.blocks[i][1]) - int(align.blocks[i][0]) + 1
                    span_sum += span
                qstart += span_sum - 1
                                
                span_sum = 0
                for b in self.align_blocks:
                    blk = align.blocks[b-1]
                    span_sum += int(blk[1]) - int(blk[0]) + 1
                qend = qstart + span_sum - 1

                #print "test-orf", qstart, qend, orf['start'], orf['end'], align.query_len

                #test if event lies within predicted orf
                if subsume([qstart, qend], [orf['start'], orf['end']]):
                    self.valid_orf = "good,%d-%d;%s...%s,%daa,%d-%d,%dnt,%.2f,%s" % (qstart, qend, orf['peptide'][:3], orf['peptide'][-3:], len(orf['peptide']), orf['start'], orf['end'], orf['end']-orf['start'], (float(orf['end'])-float(orf['start']))/float(align.query_len), orf['strand'])
                else:
                    self.valid_orf = "%d-%d;%s...%s,%daa,%d-%d,%dnt,%.2f,%s" % (qstart, qend, orf['peptide'][:3], orf['peptide'][-3:], len(orf['peptide']), orf['start'], orf['end'], orf['end']-orf['start'], (float(orf['end'])-float(orf['start']))/float(align.query_len), orf['strand'])


            else:
                self.valid_orf = "%s...%s,%daa,%d-%d,%dnt,%.2f,%s" % (orf['peptide'][:3], orf['peptide'][-3:], len(orf['peptide']), orf['start'], orf['end'], orf['end']-orf['start'], (float(orf['end'])-float(orf['start']))/float(align.contig.length), orf['strand'])

    def set_novelty(self, txts, matches=None):
        novel_events = []
        
        if self.event_type == "novel_exon" or self.event_type == "AS53":
        
            for txt in txts:
                blocks_to_delete = []
     
                for b in range(len(self.align_blocks)):
                    for e in range(len(txt.exons)):
                        novel = True
                        start_matched = False
                        end_matched = False
                    
                        if int(self.align_coords[b][0]) == int(txt.exons[e][0]):
                            start_matched = True

                        if int(self.align_coords[b][1]) == int(txt.exons[e][1]):
                            end_matched = True
                            
                        if e == 0 and end_matched:
                            novel = False
                        
                        elif e == len(txt.exons)-1 and start_matched:
                            novel = False
                        
                        elif start_matched and end_matched:
                            novel = False

                        #multiple of 3
                        apart = None
                        if self.event_type == "retained_intron":
                            apart = int(self.exon_coords[1][0]) - 1 - int(self.exon_coords[0][1])
                            self.x3 = self.multiple_3(apart)

                        if not novel:
                            blocks_to_delete.append(b)
                            break

                if blocks_to_delete:
                    for b in sorted(blocks_to_delete, reverse=True):
                        del self.align_blocks[b]
                        del self.align_coords[b]

            if not self.align_blocks:
                self.novel = False

            #print self.event_type, self.contig, self.transcript, "NOVEL:",novel

        elif self.event_type == "novel_utr":
            #print "utr", self.contig, self.transcript, self.align_blocks

            if matches:
                novel_blocks = dict((b, True) for b in self.align_blocks)
                #print "utr",novel_blocks

                for b in self.align_blocks:
                    for match in matches:
                        if match.txt.name != self.transcript:
                            if b in match.matched_blocks:
                                #print "utr %s,%s,%s %s" % (self.transcript, b, self.match_result.txt.strand, match.txt.name), match.matched_blocks, ",", match.txt.strand
                                novel_blocks[b] = False
                                #print "utr",novel_blocks
                                #print "utr done1"
                                break

                    if not True in novel_blocks.values():
                        self.novel = False
                        break

                if False in novel_blocks.values():
                    for blk in [b for b in novel_blocks.keys() if not novel_blocks[b]]:
                        self.align_blocks.remove(blk)

        elif self.event_type == "retained_intron":
            #if orignial covers more than 1 intron(multi), don't report it, dissect it into individuals and report
            multi = False
            if int(self.exons[-1]) - int(self.exons[0]) > 1:
                multi = True
                self.novel = False
            
            #go through each retained intron (reported can be multipe)
            for i in range(len(self.exon_coords)-1):
                retained_intron = [int(self.exon_coords[i][1]), int(self.exon_coords[i+1][0])]
                
                middle_exons = {}
                txts_ordered = []
                for txt in txts:
                    exons_txt = []
                    for j in range(len(txt.exons)):
                        exon = txt.exons[j]

                        #terminal exon, require subsume
                        if j == 0 or j == len(txt.exons)-1:
                            if subsume(retained_intron, exon):
                                exons_txt.append(exon) 
                        #middle exons, require just overlap
                        elif overlap(exon, retained_intron):
                            exons_txt.append(exon)

                    if exons_txt:
                        middle_exons[txt] = exons_txt
                        txts_ordered.append(txt)

                #only time when original event is novel WITHOUT testing is when it's a single ri and it is clear of overlapping exons
                if len(middle_exons.keys()) == 0 and not multi:
                    self.novel = True
                
                else:
                    self.novel = False
    
                    #print "orginall", retained_intron

                    t = [a.name for a in middle_exons.keys()]
                    #print "middle1", t
                    
                    #print "middle2", middle_exons.values()
                    #b = [middle_exons[a] for a in middle_exons.keys() if "cSep07" in a.name]
                    #print "middle3", b
                    #for t in middle_exons.keys():
                    #    print "middle2", t.name, middle_exons[t]

                    #substract overlapping exons
                    #print "middle3", retained_intron, middle_exons.values(), len(middle_exons.values()), len(middle_exons.values()[0])
                    if middle_exons.values() and middle_exons.values()[0]:
                        true_retained_intron = subtract(retained_intron, middle_exons.values())
                    else:
                        true_retained_intron = [retained_intron]
                

                    #print "true", true_retained_intron

                    #if there is still some intron left after subtraction
                    if true_retained_intron:
                        #create new events
                        for ri in true_retained_intron:
                            event = {'contig': self.contig, 'chrom':self.chrom, 'align_blocks':self.align_blocks, 'align_coords':self.align_coords, 'event_type':self.event_type, 'novel':True}

                            #determine flanking exons, and transcript by frequency of flanking coordinates
                            flanks = {}
                            for txt in txts_ordered:
                                for i in range(len(txt.exons)-1):
                                    left = txt.exons[i]
                                    right = txt.exons[i+1]
                                
                                    if left[1]+1 == ri[0] and right[0]-1 == ri[1]:
                                        #print "ri flanks", txt.full_name(), left, right, ri
                                        coord = ",".join((str(left[0]), str(left[1]), str(right[0]), str(right[1])))
                                        if not flanks.has_key(coord):
                                            flanks[coord] = [[txt,i,i+1]]
                                        else:
                                            flanks[coord].append([txt,i,i+1])

                            #print "flanks", flanks

                            if len(flanks.keys()) > 0:
                                #use the most commom frequent exons
                                flanks_sorted = flanks.keys()
                                flanks_sorted.sort(lambda x,y: len(flanks[y])-len(flanks[x]))

                                #print "flanks sorted", flanks_sorted

                                #use the orginally assigned transcript if possible
                                same_txt = False
                                exon_coords = flanks_sorted[0].split(',')
                                exons = []
                                for txt,e1,e2 in flanks[flanks_sorted[0]]:
                                    if txt.full_name() == self.transcript:
                                        same_txt = True
                                    
                                        event['transcript'] = txt.full_name()
                                        event['exons'] = [e1+1,e2+1]
                                        event['exon_coords'] = [exon_coords[:2], exon_coords[2:]]
                                        event['txt'] = txt
                                        break

                                if not same_txt:
                                    txt,e1,e2 = flanks[flanks_sorted[0]][0]
     
                                    #print "ri new", txt.full_name(), txt.exons, exons, exon_coords
                                    event['transcript'] = txt.full_name()
                                    event['exons'] = [e1+1,e2+1]
                                    event['exon_coords'] = [exon_coords[:2], exon_coords[2:]]
                                    event['txt'] = txt

                                novel_events.append(event)

        elif self.event_type == "skipped_exon":

            for txt in txts:
                for e in range(len(txt.exons)-1):
                    intron_span = [int(txt.exons[e][1])+1, int(txt.exons[e+1][0])-1]
                    #something does not have align_coords?
                    if self.align_coords and self.exon_coords:
                        if not subsume([self.align_coords[0][0], self.align_coords[-1][1]], intron_span) and subsume([self.exon_coords[0][0], self.exon_coords[-1][1]], intron_span):
                            self.novel = False
                            break
                        
                if not self.novel:
                    break

        elif self.event_type == "novel_intron":
            novel_intron_span = [int(self.align_coords[0][1])+1, int(self.align_coords[1][0])-1]
            novel_intron_size = int(self.align_coords[1][0]) - 1 - int(self.align_coords[0][1])

            for txt in txts:
                for e in range(len(txt.exons)-1):
                    intron_span = [int(txt.exons[e][1])+1, int(txt.exons[e+1][0])-1]

                    if novel_intron_span[0] == intron_span[0] and novel_intron_span[1] == intron_span[1]:
                        self.novel = False
                        break

                if not self.novel:
                    break

        elif self.event_type == "AS5":
            for txt in txts:
                for exon in txt.exons:
                    if int(self.align_coords[0]) == int(exon[0]):
                        self.novel = False
                        break

                if not self.novel:
                    break

        elif self.event_type == "AS3":
            for txt in txts:
                for exon in txt.exons:
                    if int(self.align_coords[1]) == int(exon[1]):
                        self.novel = False
                        break

                if not self.novel:
                    break

        return novel_events
           
import sys
sys.path.append('/home/rchiu/python/biopython-1.52')
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC

def best_orf(sequence):
    seq = Seq(sequence)
    orfs_sorted = find_orfs(seq, 1)

    if orfs_sorted:
        best_orf = orfs_sorted[0]
        return {'peptide':best_orf[3], 'start':best_orf[0], 'end':best_orf[1], 'strand':best_orf[2]}
    else:
        return None

def find_orfs2(sequence, min_protein_length=1, orient=None):
    seq = Seq(sequence)
    
    orfs = []
    seq_len = len(seq)
    #print "orient", orient
    for strand, nuc in [(+1, seq), (-1, seq.reverse_complement())] :
        if orient:
            if orient == "-" and strand == +1:
                continue
            if orient == "+" and strand == -1:
                continue
        
        for frame in range(3) :
            trans = str(nuc[frame:].translate())
            trans_len = len(trans)

            aa_start = 0
            aa_end = 0
            while aa_start < trans_len :
                aa_end = trans.find("*", aa_start)

                if aa_end == -1 :
                    aa_end = trans_len - 1

                if aa_end-aa_start >= min_protein_length :
                    if strand == 1 :
                        start = frame + aa_start*3
                        end = min(seq_len-1, frame+aa_end*3+3 -1)

                    else :
                        end = seq_len - 1 - frame - aa_start*3
                        start = end + 1 - (aa_end - aa_start)*3 - 3

                    orfs.append((start, end, strand,
                                       trans[aa_start:aa_end]))
                aa_start = aa_end+1
                
    orfs.sort(lambda x,y: len(y[3])-len(x[3]))

##    for o in orfs:
##        print "orf",o
##    print "done"
    return orfs

def find_orfs(seq, min_protein_length=1, orient=None):
    orfs = []
    seq_len = len(seq)

    for strand, nuc in [(+1, seq), (-1, seq.reverse_complement())] :
        
        for frame in range(3) :
            trans = str(nuc[frame:].translate())
            trans_len = len(trans)

            aa_start = 0
            aa_end = 0
            while aa_start < trans_len :
                aa_end = trans.find("*", aa_start)

                if aa_end == -1 :
                    aa_end = trans_len - 1

                if aa_end-aa_start >= min_protein_length :
                    if strand == 1 :
                        start = frame + aa_start*3
                        end = min(seq_len-1, frame+aa_end*3+3 -1)

                    else :
                        end = seq_len - 1 - frame - aa_start*3
                        start = end + 1 - (aa_end - aa_start)*3 - 3

                    orfs.append((start, end, strand,
                                       trans[aa_start:aa_end]))
                aa_start = aa_end+1
                
    orfs.sort(lambda x,y: len(y[3])-len(x[3]))

    return orfs


    
