#!/usr/bin/perl

=head1 NAME

calcPolyAinTails.pl - calculates the length of polyAs in sequences

=head1 SYNOPSYS

  % calcPolyAinTails.pl -i INPUT

=head1 DESCRIPTION

Reads a FASTA file and calculates the maximum poly-A window.


=head2 NOTES

This counts the number of pure A or T's at the end of a sequence and is very crude.
There are many transcripts with long stretches of A's but they contain other bases as well.
Probably the best method would look at motifs using a HMM method which I did not have time for.

This is still a work in progress.

=head1 COMMAND-LINE OPTIONS

Options can be abbreviated.  For example, you can use -s for
    --input            Input FASTA file
                         Default = STDIN
    --t                Count leading polyT as well
    --all              Include all entries even if no polyA tail
    --offset           Offset the tail number
    --quiet            Prints out extra information on tables and results
                         Default = 0


=head1 SEE ALSO

=head1 AUTHOR

Mark Okada, hmokada@sfu.ca

Copyright (c) 2010 Simon Fraser University

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.  See DISCLAIMER.txt for
disclaimers of warranty.

=cut

use strict;
use FindBin;

use lib "$FindBin::RealBin/../lib";

use Digest::MD5 qw(md5_hex);

use Data::Dumper;
use Getopt::Long;
use Config::General;
use vars qw(%OPT %CONF);

#default vals
%OPT = (win=>50,quiet => 0);

GetOptions(\%OPT,
	   "input=s",
	   "offset=i",
	   "t",
	   "quiet+");


open (FH, $OPT{input}) if $OPT{input};
my $fh = -e $OPT{input} ? *FH : *STDIN;


my $header;
my @seq;

while (<$fh>) {
    chomp;
    
    if (substr($_,0,1) eq ">") {
        proc_entry($header,\@seq) if $header;
        @seq = ();
        $header = $_;
    } else {
        push @seq, $_;
    }
    
    
}

if ($header) {
    proc_entry($header, \@seq);
}


close FH;


sub proc_entry {
    my $header = shift;
    my $seqs = shift;
    
    my ($tail, $head);
    
    my $seq = uc join("", @$seqs);
    ($tail) = $seq =~ /(A*)$/;
    ($head) = $seq =~ /^(T*)/ if $OPT{t};
    
    #print "$header\n$seq\n";
    
    
    my $alength = length($tail) + $OPT{offset};
    my $tlength = length($head) + $OPT{offset};
    
    
    if ($OPT{t}) {
	return if !$OPT{all} && $alength==0 && $tlength==0;
	print join("\t",
		   $header,
		   $alength,
		   $tlength),"\n";
	
    } else {
	return if ! $OPT{all} && $alength==0;
	print join("\t",
		   $header,
		   $alength),"\n";
    }
}
1;
