class SNV:
    method = None
    snv_type = None
    snv_len = None
    var = None
    var_start = None
    var_end = None
    var_seq = None
    var_len = None
    ref = None
    ref_start = None
    ref_end = None
    ref_seq = None
    ref_strand = None
    strand = None
    from_end = None
    
    def __init__(self, method, snv_type, ref, ref_start, ref_end, ref_seq, ref_strand, var, var_start, var_end, var_seq, var_len=None):
        self.method = method
        self.snv_type = snv_type
        self.var = var
        self.var_start = var_start
        self.var_end = var_end
        self.var_seq = var_seq
        self.ref = ref
        self.ref_start = ref_start
        self.ref_end = ref_end
        self.ref_seq = ref_seq
        self.ref_strand = ref_strand
        self.snv_len = len(ref_seq)

        if self.ref_strand:
            self.strand = "+"
        else:
            self.strand = "-"

    def gff(self, type):
        "output gff format"

        if 'chr' in self.ref:
            chrom = self.ref
        else:
            chrom = 'chr' + self.ref

        var_shortened = self.var.split(" ")[0]

        if self.snv_type.upper() == "SNV":
            label_last = self.ref_seq.upper() + "->" + self.var_seq.upper()
        else:
            label_last = self.snv_type.upper()
        label = ":".join([var_shortened, chrom, str(self.ref_start), label_last])

        out = ""

        out = "\t".join([chrom, self.method, type, str(self.ref_start), str(self.ref_end), ".", self.strand, ".", label])
        return out + "\n"


    def details(self):
        return "%s %s:%d-%d %s:%d-%d %d %s->%s %s" % (self.snv_type, self.ref, self.ref_start, self.ref_end, self.var, self.var_start, self.var_end, self.snv_len, self.ref_seq, self.var_seq, self.strand)

    def tab(self):
        "tabular summary of snv"
        
        if 'chr' in self.ref:
            chrom = self.ref
        else:
            chrom = 'chr' + self.ref

        var_shortened = self.var.split(" ")[0]

        change = ""
        if self.snv_type.upper() == "SNV":
            change = self.ref_seq.upper() + "->" + self.var_seq.upper()
        elif self.snv_type.upper() == "DEL":
            change = self.ref_seq.lower()
        elif self.snv_type.upper() == "INS":
            change = self.var_seq.upper()

        return "\t".join([self.snv_type, chrom, str(self.ref_start), str(self.ref_end), self.strand, var_shortened, self.var_len, str(self.var_start), str(self.var_end), str(self.snv_len), change, str(self.from_end)]) + "\n"

        
